# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared_params.ledger_transaction_create_request import LedgerTransactionCreateRequest

__all__ = ["ReturnCreateParams", "Corrections"]


class ReturnCreateParams(TypedDict, total=False):
    returnable_id: Required[Optional[str]]
    """The ID of the object being returned or `null`."""

    returnable_type: Required[Literal["incoming_payment_detail"]]
    """The type of object being returned.

    Currently, this may only be incoming_payment_detail.
    """

    additional_information: Optional[str]
    """Some returns may include additional information from the bank.

    In these cases, this string will be present.
    """

    code: Optional[
        Literal[
            "901",
            "902",
            "903",
            "904",
            "905",
            "907",
            "908",
            "909",
            "910",
            "911",
            "912",
            "914",
            "C01",
            "C02",
            "C03",
            "C05",
            "C06",
            "C07",
            "C08",
            "C09",
            "C13",
            "C14",
            "R01",
            "R02",
            "R03",
            "R04",
            "R05",
            "R06",
            "R07",
            "R08",
            "R09",
            "R10",
            "R11",
            "R12",
            "R13",
            "R14",
            "R15",
            "R16",
            "R17",
            "R18",
            "R19",
            "R20",
            "R21",
            "R22",
            "R23",
            "R24",
            "R25",
            "R26",
            "R27",
            "R28",
            "R29",
            "R30",
            "R31",
            "R32",
            "R33",
            "R34",
            "R35",
            "R36",
            "R37",
            "R38",
            "R39",
            "R40",
            "R41",
            "R42",
            "R43",
            "R44",
            "R45",
            "R46",
            "R47",
            "R50",
            "R51",
            "R52",
            "R53",
            "R61",
            "R62",
            "R67",
            "R68",
            "R69",
            "R70",
            "R71",
            "R72",
            "R73",
            "R74",
            "R75",
            "R76",
            "R77",
            "R80",
            "R81",
            "R82",
            "R83",
            "R84",
            "R85",
            "currencycloud",
        ]
    ]
    """The return code. For ACH returns, this is the required ACH return code."""

    corrections: Optional[Corrections]
    """Only relevant for ACH NOC returns.

    This is an object containing all of the new and corrected information provided
    by the bank that was previously incorrect on the original outgoing payment.
    """

    data: Optional[object]
    """The raw data from the return file that we get from the bank."""

    date_of_death: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """
    If the return code is `R14` or `R15` this is the date the deceased counterparty
    passed away.
    """

    ledger_transaction: LedgerTransactionCreateRequest
    """Specifies a ledger transaction object that will be created with the return.

    If the ledger transaction cannot be created, then the return creation will fail.
    The resulting ledger transaction will mirror the status of the return.
    """

    reason: Optional[str]
    """An optional description of the reason for the return.

    This is for internal usage and will not be transmitted to the bank.”
    """

    reconciliation_status: Literal["unreconciled", "tentatively_reconciled", "reconciled"]
    """One of `unreconciled`, `tentatively_reconciled` or `reconciled`."""


class Corrections(TypedDict, total=False):
    """Only relevant for ACH NOC returns.

    This is an object containing all of the new and corrected information provided by the bank that was previously incorrect on the original outgoing payment.
    """

    account_number: Optional[str]
    """
    The updated account number that should replace the one originally used on the
    outgoing payment.
    """

    company_id: Optional[str]
    """
    The updated company ID that should replace the one originally used on the
    outgoing payment.
    """

    company_name: Optional[str]
    """
    The updated company name that should replace the one originally used on the
    outgoing payment.
    """

    individual_identification_number: Optional[str]
    """
    The updated individual identification number that should replace the one
    originally used on the outgoing payment.
    """

    routing_number: Optional[str]
    """
    The updated routing number that should replace the one originally used on the
    outgoing payment.
    """

    transaction_code: Optional[str]
    """
    The updated account type code that should replace the one originally used on the
    outgoing payment.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import date
from typing_extensions import Literal

import httpx

from .. import _legacy_response
from ..types import return_list_params, return_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.return_object import ReturnObject
from ..types.shared_params.ledger_transaction_create_request import LedgerTransactionCreateRequest

__all__ = ["Returns", "AsyncReturns"]


class Returns(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ReturnsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return ReturnsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReturnsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return ReturnsWithStreamingResponse(self)

    def create(
        self,
        *,
        returnable_id: Optional[str],
        returnable_type: Literal["incoming_payment_detail"],
        additional_information: Optional[str] | Omit = omit,
        code: Optional[
            Literal[
                "901",
                "902",
                "903",
                "904",
                "905",
                "907",
                "908",
                "909",
                "910",
                "911",
                "912",
                "914",
                "C01",
                "C02",
                "C03",
                "C05",
                "C06",
                "C07",
                "C08",
                "C09",
                "C13",
                "C14",
                "R01",
                "R02",
                "R03",
                "R04",
                "R05",
                "R06",
                "R07",
                "R08",
                "R09",
                "R10",
                "R11",
                "R12",
                "R13",
                "R14",
                "R15",
                "R16",
                "R17",
                "R18",
                "R19",
                "R20",
                "R21",
                "R22",
                "R23",
                "R24",
                "R25",
                "R26",
                "R27",
                "R28",
                "R29",
                "R30",
                "R31",
                "R32",
                "R33",
                "R34",
                "R35",
                "R36",
                "R37",
                "R38",
                "R39",
                "R40",
                "R41",
                "R42",
                "R43",
                "R44",
                "R45",
                "R46",
                "R47",
                "R50",
                "R51",
                "R52",
                "R53",
                "R61",
                "R62",
                "R67",
                "R68",
                "R69",
                "R70",
                "R71",
                "R72",
                "R73",
                "R74",
                "R75",
                "R76",
                "R77",
                "R80",
                "R81",
                "R82",
                "R83",
                "R84",
                "R85",
                "currencycloud",
            ]
        ]
        | Omit = omit,
        corrections: Optional[return_create_params.Corrections] | Omit = omit,
        data: Optional[object] | Omit = omit,
        date_of_death: Union[str, date, None] | Omit = omit,
        ledger_transaction: LedgerTransactionCreateRequest | Omit = omit,
        reason: Optional[str] | Omit = omit,
        reconciliation_status: Literal["unreconciled", "tentatively_reconciled", "reconciled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> ReturnObject:
        """
        Create a return.

        Args:
          returnable_id: The ID of the object being returned or `null`.

          returnable_type: The type of object being returned. Currently, this may only be
              incoming_payment_detail.

          additional_information: Some returns may include additional information from the bank. In these cases,
              this string will be present.

          code: The return code. For ACH returns, this is the required ACH return code.

          corrections: Only relevant for ACH NOC returns. This is an object containing all of the new
              and corrected information provided by the bank that was previously incorrect on
              the original outgoing payment.

          data: The raw data from the return file that we get from the bank.

          date_of_death: If the return code is `R14` or `R15` this is the date the deceased counterparty
              passed away.

          ledger_transaction: Specifies a ledger transaction object that will be created with the return. If
              the ledger transaction cannot be created, then the return creation will fail.
              The resulting ledger transaction will mirror the status of the return.

          reason: An optional description of the reason for the return. This is for internal usage
              and will not be transmitted to the bank.”

          reconciliation_status: One of `unreconciled`, `tentatively_reconciled` or `reconciled`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/returns",
            body=maybe_transform(
                {
                    "returnable_id": returnable_id,
                    "returnable_type": returnable_type,
                    "additional_information": additional_information,
                    "code": code,
                    "corrections": corrections,
                    "data": data,
                    "date_of_death": date_of_death,
                    "ledger_transaction": ledger_transaction,
                    "reason": reason,
                    "reconciliation_status": reconciliation_status,
                },
                return_create_params.ReturnCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=ReturnObject,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReturnObject:
        """
        Get a single return.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/api/returns/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReturnObject,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | Omit = omit,
        counterparty_id: str | Omit = omit,
        internal_account_id: str | Omit = omit,
        per_page: int | Omit = omit,
        returnable_id: str | Omit = omit,
        returnable_type: Literal["incoming_payment_detail", "payment_order", "return", "reversal"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPage[ReturnObject]:
        """
        Get a list of returns.

        Args:
          counterparty_id: Specify `counterparty_id` if you wish to see returns that occurred with a
              specific counterparty.

          internal_account_id: Specify `internal_account_id` if you wish to see returns to/from a specific
              account.

          returnable_id: The ID of a valid returnable. Must be accompanied by `returnable_type`.

          returnable_type: One of `payment_order`, `reversal`, or `incoming_payment_detail`. Must be
              accompanied by `returnable_id`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/returns",
            page=SyncPage[ReturnObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "counterparty_id": counterparty_id,
                        "internal_account_id": internal_account_id,
                        "per_page": per_page,
                        "returnable_id": returnable_id,
                        "returnable_type": returnable_type,
                    },
                    return_list_params.ReturnListParams,
                ),
            ),
            model=ReturnObject,
        )


class AsyncReturns(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncReturnsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReturnsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReturnsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncReturnsWithStreamingResponse(self)

    async def create(
        self,
        *,
        returnable_id: Optional[str],
        returnable_type: Literal["incoming_payment_detail"],
        additional_information: Optional[str] | Omit = omit,
        code: Optional[
            Literal[
                "901",
                "902",
                "903",
                "904",
                "905",
                "907",
                "908",
                "909",
                "910",
                "911",
                "912",
                "914",
                "C01",
                "C02",
                "C03",
                "C05",
                "C06",
                "C07",
                "C08",
                "C09",
                "C13",
                "C14",
                "R01",
                "R02",
                "R03",
                "R04",
                "R05",
                "R06",
                "R07",
                "R08",
                "R09",
                "R10",
                "R11",
                "R12",
                "R13",
                "R14",
                "R15",
                "R16",
                "R17",
                "R18",
                "R19",
                "R20",
                "R21",
                "R22",
                "R23",
                "R24",
                "R25",
                "R26",
                "R27",
                "R28",
                "R29",
                "R30",
                "R31",
                "R32",
                "R33",
                "R34",
                "R35",
                "R36",
                "R37",
                "R38",
                "R39",
                "R40",
                "R41",
                "R42",
                "R43",
                "R44",
                "R45",
                "R46",
                "R47",
                "R50",
                "R51",
                "R52",
                "R53",
                "R61",
                "R62",
                "R67",
                "R68",
                "R69",
                "R70",
                "R71",
                "R72",
                "R73",
                "R74",
                "R75",
                "R76",
                "R77",
                "R80",
                "R81",
                "R82",
                "R83",
                "R84",
                "R85",
                "currencycloud",
            ]
        ]
        | Omit = omit,
        corrections: Optional[return_create_params.Corrections] | Omit = omit,
        data: Optional[object] | Omit = omit,
        date_of_death: Union[str, date, None] | Omit = omit,
        ledger_transaction: LedgerTransactionCreateRequest | Omit = omit,
        reason: Optional[str] | Omit = omit,
        reconciliation_status: Literal["unreconciled", "tentatively_reconciled", "reconciled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> ReturnObject:
        """
        Create a return.

        Args:
          returnable_id: The ID of the object being returned or `null`.

          returnable_type: The type of object being returned. Currently, this may only be
              incoming_payment_detail.

          additional_information: Some returns may include additional information from the bank. In these cases,
              this string will be present.

          code: The return code. For ACH returns, this is the required ACH return code.

          corrections: Only relevant for ACH NOC returns. This is an object containing all of the new
              and corrected information provided by the bank that was previously incorrect on
              the original outgoing payment.

          data: The raw data from the return file that we get from the bank.

          date_of_death: If the return code is `R14` or `R15` this is the date the deceased counterparty
              passed away.

          ledger_transaction: Specifies a ledger transaction object that will be created with the return. If
              the ledger transaction cannot be created, then the return creation will fail.
              The resulting ledger transaction will mirror the status of the return.

          reason: An optional description of the reason for the return. This is for internal usage
              and will not be transmitted to the bank.”

          reconciliation_status: One of `unreconciled`, `tentatively_reconciled` or `reconciled`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/returns",
            body=await async_maybe_transform(
                {
                    "returnable_id": returnable_id,
                    "returnable_type": returnable_type,
                    "additional_information": additional_information,
                    "code": code,
                    "corrections": corrections,
                    "data": data,
                    "date_of_death": date_of_death,
                    "ledger_transaction": ledger_transaction,
                    "reason": reason,
                    "reconciliation_status": reconciliation_status,
                },
                return_create_params.ReturnCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=ReturnObject,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReturnObject:
        """
        Get a single return.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/api/returns/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReturnObject,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | Omit = omit,
        counterparty_id: str | Omit = omit,
        internal_account_id: str | Omit = omit,
        per_page: int | Omit = omit,
        returnable_id: str | Omit = omit,
        returnable_type: Literal["incoming_payment_detail", "payment_order", "return", "reversal"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ReturnObject, AsyncPage[ReturnObject]]:
        """
        Get a list of returns.

        Args:
          counterparty_id: Specify `counterparty_id` if you wish to see returns that occurred with a
              specific counterparty.

          internal_account_id: Specify `internal_account_id` if you wish to see returns to/from a specific
              account.

          returnable_id: The ID of a valid returnable. Must be accompanied by `returnable_type`.

          returnable_type: One of `payment_order`, `reversal`, or `incoming_payment_detail`. Must be
              accompanied by `returnable_id`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/returns",
            page=AsyncPage[ReturnObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "counterparty_id": counterparty_id,
                        "internal_account_id": internal_account_id,
                        "per_page": per_page,
                        "returnable_id": returnable_id,
                        "returnable_type": returnable_type,
                    },
                    return_list_params.ReturnListParams,
                ),
            ),
            model=ReturnObject,
        )


class ReturnsWithRawResponse:
    def __init__(self, returns: Returns) -> None:
        self._returns = returns

        self.create = _legacy_response.to_raw_response_wrapper(
            returns.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            returns.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            returns.list,
        )


class AsyncReturnsWithRawResponse:
    def __init__(self, returns: AsyncReturns) -> None:
        self._returns = returns

        self.create = _legacy_response.async_to_raw_response_wrapper(
            returns.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            returns.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            returns.list,
        )


class ReturnsWithStreamingResponse:
    def __init__(self, returns: Returns) -> None:
        self._returns = returns

        self.create = to_streamed_response_wrapper(
            returns.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            returns.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            returns.list,
        )


class AsyncReturnsWithStreamingResponse:
    def __init__(self, returns: AsyncReturns) -> None:
        self._returns = returns

        self.create = async_to_streamed_response_wrapper(
            returns.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            returns.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            returns.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import FileTypes

__all__ = ["DocumentCreateParams"]


class DocumentCreateParams(TypedDict, total=False):
    file: Required[FileTypes]

    document_type: str
    """A category given to the document, can be `null`."""

    documentable_id: str
    """The unique identifier for the associated object."""

    documentable_type: Literal[
        "connections",
        "counterparties",
        "expected_payments",
        "external_accounts",
        "identifications",
        "incoming_payment_details",
        "internal_accounts",
        "legal_entities",
        "organizations",
        "payment_orders",
        "transactions",
    ]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Optional
from datetime import date, datetime
from typing_extensions import Literal

from .._models import BaseModel
from .document import Document
from .shared.third_party_verification import ThirdPartyVerification
from .shared.legal_entity_industry_classification import LegalEntityIndustryClassification

__all__ = [
    "LegalEntity",
    "Addresses",
    "Address",
    "BankSettings",
    "Identifications",
    "Identification",
    "PhoneNumbers",
    "PhoneNumber",
    "Regulators",
    "Regulator",
    "WealthAndEmploymentDetails",
]


class Address(BaseModel):
    id: str

    address_types: List[Literal["business", "business_registered", "mailing", "other", "po_box", "residential"]]
    """The types of this address."""

    country: Optional[str] = None
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    line1: Optional[str] = None

    line2: Optional[str] = None

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    locality: Optional[str] = None
    """Locality or City."""

    object: str

    postal_code: Optional[str] = None
    """The postal code of the address."""

    region: Optional[str] = None
    """Region or State."""

    updated_at: datetime


Addresses = Address
"""This type is deprecated and will be removed in a future release.

Please use Address instead.
"""


class BankSettings(BaseModel):
    id: str

    backup_withholding_percentage: Optional[int] = None
    """The percentage of backup withholding to apply to the legal entity."""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    enable_backup_withholding: Optional[bool] = None
    """Whether backup withholding is enabled.

    See more here -
    https://www.irs.gov/businesses/small-businesses-self-employed/backup-withholding.
    """

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    privacy_opt_out: Optional[bool] = None
    """Cross River Bank specific setting to opt out of privacy policy."""

    regulation_o: Optional[bool] = None
    """
    It covers, among other types of insider loans, extensions of credit by a member
    bank to an executive officer, director, or principal shareholder of the member
    bank; a bank holding company of which the member bank is a subsidiary; and any
    other subsidiary of that bank holding company.
    """

    updated_at: datetime


class Identification(BaseModel):
    id: str

    created_at: datetime

    discarded_at: Optional[datetime] = None

    documents: List[Document]

    expiration_date: Optional[date] = None
    """
    The date when the Identification is no longer considered valid by the issuing
    authority.
    """

    id_type: Literal[
        "ar_cuil",
        "ar_cuit",
        "br_cnpj",
        "br_cpf",
        "ca_sin",
        "cl_run",
        "cl_rut",
        "co_cedulas",
        "co_nit",
        "drivers_license",
        "hn_id",
        "hn_rtn",
        "ie_pps",
        "in_lei",
        "kr_brn",
        "kr_crn",
        "kr_rrn",
        "passport",
        "sa_tin",
        "sa_vat",
        "us_ein",
        "us_itin",
        "us_ssn",
        "vn_tin",
    ]
    """The type of ID number."""

    issuing_country: Optional[str] = None
    """
    The ISO 3166-1 alpha-2 country code of the country that issued the
    identification
    """

    issuing_region: Optional[str] = None
    """The region in which the identifcation was issued."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    updated_at: datetime


Identifications = Identification
"""This type is deprecated and will be removed in a future release.

Please use Identification instead.
"""


class PhoneNumber(BaseModel):
    """A list of phone numbers in E.164 format."""

    phone_number: Optional[str] = None


PhoneNumbers = PhoneNumber
"""This type is deprecated and will be removed in a future release.

Please use PhoneNumber instead.
"""


class Regulator(BaseModel):
    jurisdiction: str
    """
    The country code where the regulator operates in the ISO 3166-1 alpha-2 format
    (e.g., "US", "CA", "GB").
    """

    name: str
    """Full name of the regulatory body."""

    registration_number: str
    """Registration or identification number with the regulator."""


Regulators = Regulator
"""This type is deprecated and will be removed in a future release.

Please use Regulator instead.
"""


class WealthAndEmploymentDetails(BaseModel):
    id: str

    annual_income: Optional[int] = None
    """The annual income of the individual in USD."""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    employer_country: Optional[str] = None
    """The country in which the employer is located."""

    employer_name: Optional[str] = None
    """The name of the employer."""

    employer_state: Optional[str] = None
    """The state in which the employer is located."""

    employment_status: Optional[Literal["employed", "retired", "self_employed", "student", "unemployed"]] = None
    """The employment status of the individual."""

    income_country: Optional[str] = None
    """The country in which the individual's income is earned."""

    income_source: Optional[
        Literal[
            "family_support",
            "government_benefits",
            "inheritance",
            "investments",
            "rental_income",
            "retirement",
            "salary",
            "self_employed",
        ]
    ] = None
    """The source of the individual's income."""

    income_state: Optional[str] = None
    """The state in which the individual's income is earned."""

    industry: Optional[
        Literal[
            "accounting",
            "agriculture",
            "automotive",
            "chemical_manufacturing",
            "construction",
            "educational_medical",
            "food_service",
            "finance",
            "gasoline",
            "health_stores",
            "laundry",
            "maintenance",
            "manufacturing",
            "merchant_wholesale",
            "mining",
            "performing_arts",
            "professional_non_legal",
            "public_administration",
            "publishing",
            "real_estate",
            "recreation_gambling",
            "religious_charity",
            "rental_services",
            "retail_clothing",
            "retail_electronics",
            "retail_food",
            "retail_furnishing",
            "retail_home",
            "retail_non_store",
            "retail_sporting",
            "transportation",
            "travel",
            "utilities",
        ]
    ] = None
    """The industry of the individual."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    occupation: Optional[
        Literal[
            "consulting",
            "executive",
            "finance_accounting",
            "food_services",
            "government",
            "healthcare",
            "legal_services",
            "manufacturing",
            "other",
            "sales",
            "science_engineering",
            "technology",
        ]
    ] = None
    """The occupation of the individual."""

    source_of_funds: Optional[
        Literal[
            "alimony",
            "annuity",
            "business_owner",
            "business_revenue",
            "debt_financing",
            "general_employee",
            "government_benefits",
            "homemaker",
            "inheritance_gift",
            "intercompany_loan",
            "investment",
            "investor_funding",
            "legal_settlement",
            "lottery",
            "real_estate",
            "retained_earnings_or_savings",
            "retired",
            "retirement",
            "salary",
            "sale_of_business_assets",
            "sale_of_real_estate",
            "self_employed",
            "senior_executive",
            "trust_income",
        ]
    ] = None
    """The source of the individual's funds."""

    updated_at: datetime

    wealth_source: Optional[
        Literal[
            "business_sale",
            "family_support",
            "government_benefits",
            "inheritance",
            "investments",
            "other",
            "rental_income",
            "retirement",
            "salary",
            "self_employed",
        ]
    ] = None
    """The source of the individual's wealth."""


class LegalEntity(BaseModel):
    id: str

    addresses: List[Address]
    """A list of addresses for the entity."""

    bank_settings: Optional[BankSettings] = None

    business_description: Optional[str] = None
    """A description of the business."""

    business_name: Optional[str] = None
    """The business's legal business name."""

    citizenship_country: Optional[str] = None
    """The country of citizenship for an individual."""

    compliance_details: Optional[object] = None

    country_of_incorporation: Optional[str] = None
    """
    The country code where the business is incorporated in the ISO 3166-1 alpha-2 or
    alpha-3 formats.
    """

    created_at: datetime

    date_formed: Optional[date] = None
    """A business's formation date (YYYY-MM-DD)."""

    date_of_birth: Optional[date] = None
    """An individual's date of birth (YYYY-MM-DD)."""

    discarded_at: Optional[datetime] = None

    documents: List[Document]

    doing_business_as_names: List[str]

    email: Optional[str] = None
    """The entity's primary email."""

    expected_activity_volume: Optional[int] = None
    """Monthly expected transaction volume in USD."""

    external_id: Optional[str] = None
    """An optional user-defined 180 character unique identifier."""

    first_name: Optional[str] = None
    """An individual's first name."""

    identifications: List[Identification]
    """A list of identifications for the legal entity."""

    industry_classifications: List[LegalEntityIndustryClassification]
    """A list of industry classifications for the legal entity."""

    intended_use: Optional[str] = None
    """A description of the intended use of the legal entity."""

    last_name: Optional[str] = None
    """An individual's last name."""

    legal_entity_type: Literal["business", "individual", "joint"]
    """The type of legal entity."""

    legal_structure: Optional[
        Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
    ] = None
    """The business's legal structure."""

    listed_exchange: Optional[str] = None
    """ISO 10383 market identifier code."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    middle_name: Optional[str] = None
    """An individual's middle name."""

    object: str

    operating_jurisdictions: List[str]
    """
    A list of countries where the business operates (ISO 3166-1 alpha-2 or alpha-3
    codes).
    """

    phone_numbers: List[PhoneNumber]

    politically_exposed_person: Optional[bool] = None
    """Whether the individual is a politically exposed person."""

    preferred_name: Optional[str] = None
    """An individual's preferred name."""

    prefix: Optional[str] = None
    """An individual's prefix."""

    primary_social_media_sites: List[str]
    """A list of primary social media URLs for the business."""

    regulators: Optional[List[Regulator]] = None
    """Array of regulatory bodies overseeing this institution."""

    risk_rating: Optional[Literal["low", "medium", "high"]] = None
    """The risk rating of the legal entity. One of low, medium, high."""

    service_provider_legal_entity_id: Optional[str] = None
    """The UUID of the parent legal entity in the service provider tree."""

    status: Optional[Literal["active", "denied", "pending", "suspended"]] = None
    """The activation status of the legal entity.

    One of pending, active, suspended, or denied.
    """

    suffix: Optional[str] = None
    """An individual's suffix."""

    third_party_verification: Optional[ThirdPartyVerification] = None
    """Deprecated. Use `third_party_verifications` instead."""

    third_party_verifications: List[ThirdPartyVerification]
    """A list of third-party verifications run by external vendors."""

    ticker_symbol: Optional[str] = None
    """Stock ticker symbol for publicly traded companies."""

    updated_at: datetime

    wealth_and_employment_details: Optional[WealthAndEmploymentDetails] = None

    website: Optional[str] = None
    """The entity's primary website URL."""

    legal_entity_associations: Optional[List["LegalEntityAssociation"]] = None
    """The legal entity associations and its child legal entities."""


from .legal_entity_association import LegalEntityAssociation



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import (
    Counterparty,
    CounterpartyCollectAccountResponse,
)
from modern_treasury._utils import parse_date, parse_datetime
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCounterparties:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.create(
            name="name",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.create(
            name="name",
            accounting={"type": "customer"},
            accounts=[
                {
                    "account_details": [
                        {
                            "account_number": "account_number",
                            "account_number_type": "au_number",
                        }
                    ],
                    "account_type": "base_wallet",
                    "contact_details": [
                        {
                            "contact_identifier": "contact_identifier",
                            "contact_identifier_type": "email",
                        }
                    ],
                    "external_id": "external_id",
                    "ledger_account": {
                        "currency": "currency",
                        "ledger_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "name": "name",
                        "normal_balance": "credit",
                        "currency_exponent": 0,
                        "description": "description",
                        "external_id": "external_id",
                        "ledger_account_category_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                        "ledgerable_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "ledgerable_type": "counterparty",
                        "metadata": {
                            "key": "value",
                            "foo": "bar",
                            "modern": "treasury",
                        },
                    },
                    "metadata": {
                        "key": "value",
                        "foo": "bar",
                        "modern": "treasury",
                    },
                    "name": "name",
                    "party_address": {
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "locality": "locality",
                        "postal_code": "postal_code",
                        "region": "region",
                    },
                    "party_identifier": "party_identifier",
                    "party_name": "party_name",
                    "party_type": "business",
                    "plaid_processor_token": "plaid_processor_token",
                    "routing_details": [
                        {
                            "routing_number": "routing_number",
                            "routing_number_type": "aba",
                            "payment_type": "ach",
                        }
                    ],
                }
            ],
            email="dev@stainless.com",
            external_id="external_id",
            ledger_type="customer",
            legal_entity={
                "legal_entity_type": "business",
                "addresses": [
                    {
                        "country": "country",
                        "line1": "line1",
                        "locality": "locality",
                        "postal_code": "postal_code",
                        "region": "region",
                        "address_types": ["business"],
                        "line2": "line2",
                    }
                ],
                "bank_settings": {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "backup_withholding_percentage": 0,
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "enable_backup_withholding": True,
                    "live_mode": True,
                    "object": "object",
                    "privacy_opt_out": True,
                    "regulation_o": True,
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                },
                "business_description": "business_description",
                "business_name": "business_name",
                "citizenship_country": "citizenship_country",
                "compliance_details": {},
                "connection_id": "connection_id",
                "country_of_incorporation": "country_of_incorporation",
                "date_formed": parse_date("2019-12-27"),
                "date_of_birth": parse_date("2019-12-27"),
                "documents": [
                    {
                        "document_type": "articles_of_incorporation",
                        "file_data": "file_data",
                        "filename": "filename",
                    }
                ],
                "doing_business_as_names": ["string"],
                "email": "email",
                "expected_activity_volume": 0,
                "external_id": "external_id",
                "first_name": "first_name",
                "identifications": [
                    {
                        "id_number": "id_number",
                        "id_type": "ar_cuil",
                        "documents": [
                            {
                                "document_type": "articles_of_incorporation",
                                "file_data": "file_data",
                                "filename": "filename",
                            }
                        ],
                        "expiration_date": parse_date("2019-12-27"),
                        "issuing_country": "issuing_country",
                        "issuing_region": "issuing_region",
                    }
                ],
                "industry_classifications": [
                    {
                        "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "classification_codes": ["string"],
                        "classification_type": "anzsic",
                        "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "live_mode": True,
                        "object": "object",
                        "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
                "intended_use": "intended_use",
                "last_name": "last_name",
                "legal_entity_associations": [
                    {
                        "relationship_types": ["authorized_signer"],
                        "child_legal_entity": {
                            "addresses": [
                                {
                                    "country": "country",
                                    "line1": "line1",
                                    "locality": "locality",
                                    "postal_code": "postal_code",
                                    "region": "region",
                                    "address_types": ["business"],
                                    "line2": "line2",
                                }
                            ],
                            "bank_settings": {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "backup_withholding_percentage": 0,
                                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "enable_backup_withholding": True,
                                "live_mode": True,
                                "object": "object",
                                "privacy_opt_out": True,
                                "regulation_o": True,
                                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            },
                            "business_description": "business_description",
                            "business_name": "business_name",
                            "citizenship_country": "citizenship_country",
                            "compliance_details": {},
                            "connection_id": "connection_id",
                            "country_of_incorporation": "country_of_incorporation",
                            "date_formed": parse_date("2019-12-27"),
                            "date_of_birth": parse_date("2019-12-27"),
                            "documents": [
                                {
                                    "document_type": "articles_of_incorporation",
                                    "file_data": "file_data",
                                    "filename": "filename",
                                }
                            ],
                            "doing_business_as_names": ["string"],
                            "email": "email",
                            "expected_activity_volume": 0,
                            "external_id": "external_id",
                            "first_name": "first_name",
                            "identifications": [
                                {
                                    "id_number": "id_number",
                                    "id_type": "ar_cuil",
                                    "documents": [
                                        {
                                            "document_type": "articles_of_incorporation",
                                            "file_data": "file_data",
                                            "filename": "filename",
                                        }
                                    ],
                                    "expiration_date": parse_date("2019-12-27"),
                                    "issuing_country": "issuing_country",
                                    "issuing_region": "issuing_region",
                                }
                            ],
                            "industry_classifications": [
                                {
                                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                    "classification_codes": ["string"],
                                    "classification_type": "anzsic",
                                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "live_mode": True,
                                    "object": "object",
                                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                }
                            ],
                            "intended_use": "intended_use",
                            "last_name": "last_name",
                            "legal_entity_associations": [],
                            "legal_entity_type": "business",
                            "legal_structure": "corporation",
                            "listed_exchange": "listed_exchange",
                            "metadata": {
                                "key": "value",
                                "foo": "bar",
                                "modern": "treasury",
                            },
                            "middle_name": "middle_name",
                            "operating_jurisdictions": ["string"],
                            "phone_numbers": [{"phone_number": "phone_number"}],
                            "politically_exposed_person": True,
                            "preferred_name": "preferred_name",
                            "prefix": "prefix",
                            "primary_social_media_sites": ["string"],
                            "regulators": [
                                {
                                    "jurisdiction": "jurisdiction",
                                    "name": "name",
                                    "registration_number": "registration_number",
                                }
                            ],
                            "risk_rating": "low",
                            "service_provider_legal_entity_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "suffix": "suffix",
                            "third_party_verification": {
                                "outcome": "passed",
                                "vendor": "persona",
                                "vendor_verification_id": "vendor_verification_id",
                                "verification_category": "legal_name",
                                "verification_method": "verification_method",
                                "verification_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "comment": "comment",
                            },
                            "third_party_verifications": [
                                {
                                    "outcome": "passed",
                                    "vendor": "persona",
                                    "vendor_verification_id": "vendor_verification_id",
                                    "verification_category": "legal_name",
                                    "verification_method": "verification_method",
                                    "verification_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "comment": "comment",
                                }
                            ],
                            "ticker_symbol": "ticker_symbol",
                            "wealth_and_employment_details": {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "annual_income": 0,
                                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "employer_country": "employer_country",
                                "employer_name": "employer_name",
                                "employer_state": "employer_state",
                                "employment_status": "employed",
                                "income_country": "income_country",
                                "income_source": "family_support",
                                "income_state": "income_state",
                                "industry": "accounting",
                                "live_mode": True,
                                "object": "object",
                                "occupation": "consulting",
                                "source_of_funds": "alimony",
                                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "wealth_source": "business_sale",
                            },
                            "website": "website",
                        },
                        "child_legal_entity_id": "child_legal_entity_id",
                        "ownership_percentage": 0,
                        "title": "title",
                    }
                ],
                "legal_structure": "corporation",
                "listed_exchange": "listed_exchange",
                "metadata": {
                    "key": "value",
                    "foo": "bar",
                    "modern": "treasury",
                },
                "middle_name": "middle_name",
                "operating_jurisdictions": ["string"],
                "phone_numbers": [{"phone_number": "phone_number"}],
                "politically_exposed_person": True,
                "preferred_name": "preferred_name",
                "prefix": "prefix",
                "primary_social_media_sites": ["string"],
                "regulators": [
                    {
                        "jurisdiction": "jurisdiction",
                        "name": "name",
                        "registration_number": "registration_number",
                    }
                ],
                "risk_rating": "low",
                "service_provider_legal_entity_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "suffix": "suffix",
                "third_party_verification": {
                    "outcome": "passed",
                    "vendor": "persona",
                    "vendor_verification_id": "vendor_verification_id",
                    "verification_category": "legal_name",
                    "verification_method": "verification_method",
                    "verification_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "comment": "comment",
                },
                "third_party_verifications": [
                    {
                        "outcome": "passed",
                        "vendor": "persona",
                        "vendor_verification_id": "vendor_verification_id",
                        "verification_category": "legal_name",
                        "verification_method": "verification_method",
                        "verification_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "comment": "comment",
                    }
                ],
                "ticker_symbol": "ticker_symbol",
                "wealth_and_employment_details": {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "annual_income": 0,
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "employer_country": "employer_country",
                    "employer_name": "employer_name",
                    "employer_state": "employer_state",
                    "employment_status": "employed",
                    "income_country": "income_country",
                    "income_source": "family_support",
                    "income_state": "income_state",
                    "industry": "accounting",
                    "live_mode": True,
                    "object": "object",
                    "occupation": "consulting",
                    "source_of_funds": "alimony",
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "wealth_source": "business_sale",
                },
                "website": "website",
            },
            legal_entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            send_remittance_advice=True,
            taxpayer_identifier="taxpayer_identifier",
            verification_status="verification_status",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.counterparties.with_raw_response.create(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.counterparties.with_streaming_response.create(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = response.parse()
            assert_matches_type(Counterparty, counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.retrieve(
            "id",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.counterparties.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.counterparties.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = response.parse()
            assert_matches_type(Counterparty, counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.counterparties.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.update(
            id="id",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.update(
            id="id",
            email="dev@stainless.com",
            external_id="external_id",
            legal_entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={"foo": "string"},
            name="name",
            send_remittance_advice=True,
            taxpayer_identifier="taxpayer_identifier",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: ModernTreasury) -> None:
        response = client.counterparties.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: ModernTreasury) -> None:
        with client.counterparties.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = response.parse()
            assert_matches_type(Counterparty, counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.counterparties.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.list()
        assert_matches_type(SyncPage[Counterparty], counterparty, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.list(
            after_cursor="after_cursor",
            created_at_lower_bound=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_upper_bound=parse_datetime("2019-12-27T18:11:19.117Z"),
            email="dev@stainless.com",
            external_id="external_id",
            legal_entity_id="legal_entity_id",
            metadata={"foo": "string"},
            name="name",
            per_page=0,
        )
        assert_matches_type(SyncPage[Counterparty], counterparty, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.counterparties.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(SyncPage[Counterparty], counterparty, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.counterparties.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = response.parse()
            assert_matches_type(SyncPage[Counterparty], counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.delete(
            "id",
        )
        assert counterparty is None

    @parametrize
    def test_raw_response_delete(self, client: ModernTreasury) -> None:
        response = client.counterparties.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert counterparty is None

    @parametrize
    def test_streaming_response_delete(self, client: ModernTreasury) -> None:
        with client.counterparties.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = response.parse()
            assert counterparty is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.counterparties.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_collect_account(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.collect_account(
            id="id",
            direction="credit",
        )
        assert_matches_type(CounterpartyCollectAccountResponse, counterparty, path=["response"])

    @parametrize
    def test_method_collect_account_with_all_params(self, client: ModernTreasury) -> None:
        counterparty = client.counterparties.collect_account(
            id="id",
            direction="credit",
            custom_redirect="https://example.com",
            fields=["name"],
            send_email=True,
        )
        assert_matches_type(CounterpartyCollectAccountResponse, counterparty, path=["response"])

    @parametrize
    def test_raw_response_collect_account(self, client: ModernTreasury) -> None:
        response = client.counterparties.with_raw_response.collect_account(
            id="id",
            direction="credit",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(CounterpartyCollectAccountResponse, counterparty, path=["response"])

    @parametrize
    def test_streaming_response_collect_account(self, client: ModernTreasury) -> None:
        with client.counterparties.with_streaming_response.collect_account(
            id="id",
            direction="credit",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = response.parse()
            assert_matches_type(CounterpartyCollectAccountResponse, counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_collect_account(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.counterparties.with_raw_response.collect_account(
                id="",
                direction="credit",
            )


class TestAsyncCounterparties:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.create(
            name="name",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.create(
            name="name",
            accounting={"type": "customer"},
            accounts=[
                {
                    "account_details": [
                        {
                            "account_number": "account_number",
                            "account_number_type": "au_number",
                        }
                    ],
                    "account_type": "base_wallet",
                    "contact_details": [
                        {
                            "contact_identifier": "contact_identifier",
                            "contact_identifier_type": "email",
                        }
                    ],
                    "external_id": "external_id",
                    "ledger_account": {
                        "currency": "currency",
                        "ledger_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "name": "name",
                        "normal_balance": "credit",
                        "currency_exponent": 0,
                        "description": "description",
                        "external_id": "external_id",
                        "ledger_account_category_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                        "ledgerable_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "ledgerable_type": "counterparty",
                        "metadata": {
                            "key": "value",
                            "foo": "bar",
                            "modern": "treasury",
                        },
                    },
                    "metadata": {
                        "key": "value",
                        "foo": "bar",
                        "modern": "treasury",
                    },
                    "name": "name",
                    "party_address": {
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "locality": "locality",
                        "postal_code": "postal_code",
                        "region": "region",
                    },
                    "party_identifier": "party_identifier",
                    "party_name": "party_name",
                    "party_type": "business",
                    "plaid_processor_token": "plaid_processor_token",
                    "routing_details": [
                        {
                            "routing_number": "routing_number",
                            "routing_number_type": "aba",
                            "payment_type": "ach",
                        }
                    ],
                }
            ],
            email="dev@stainless.com",
            external_id="external_id",
            ledger_type="customer",
            legal_entity={
                "legal_entity_type": "business",
                "addresses": [
                    {
                        "country": "country",
                        "line1": "line1",
                        "locality": "locality",
                        "postal_code": "postal_code",
                        "region": "region",
                        "address_types": ["business"],
                        "line2": "line2",
                    }
                ],
                "bank_settings": {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "backup_withholding_percentage": 0,
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "enable_backup_withholding": True,
                    "live_mode": True,
                    "object": "object",
                    "privacy_opt_out": True,
                    "regulation_o": True,
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                },
                "business_description": "business_description",
                "business_name": "business_name",
                "citizenship_country": "citizenship_country",
                "compliance_details": {},
                "connection_id": "connection_id",
                "country_of_incorporation": "country_of_incorporation",
                "date_formed": parse_date("2019-12-27"),
                "date_of_birth": parse_date("2019-12-27"),
                "documents": [
                    {
                        "document_type": "articles_of_incorporation",
                        "file_data": "file_data",
                        "filename": "filename",
                    }
                ],
                "doing_business_as_names": ["string"],
                "email": "email",
                "expected_activity_volume": 0,
                "external_id": "external_id",
                "first_name": "first_name",
                "identifications": [
                    {
                        "id_number": "id_number",
                        "id_type": "ar_cuil",
                        "documents": [
                            {
                                "document_type": "articles_of_incorporation",
                                "file_data": "file_data",
                                "filename": "filename",
                            }
                        ],
                        "expiration_date": parse_date("2019-12-27"),
                        "issuing_country": "issuing_country",
                        "issuing_region": "issuing_region",
                    }
                ],
                "industry_classifications": [
                    {
                        "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "classification_codes": ["string"],
                        "classification_type": "anzsic",
                        "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "live_mode": True,
                        "object": "object",
                        "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    }
                ],
                "intended_use": "intended_use",
                "last_name": "last_name",
                "legal_entity_associations": [
                    {
                        "relationship_types": ["authorized_signer"],
                        "child_legal_entity": {
                            "addresses": [
                                {
                                    "country": "country",
                                    "line1": "line1",
                                    "locality": "locality",
                                    "postal_code": "postal_code",
                                    "region": "region",
                                    "address_types": ["business"],
                                    "line2": "line2",
                                }
                            ],
                            "bank_settings": {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "backup_withholding_percentage": 0,
                                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "enable_backup_withholding": True,
                                "live_mode": True,
                                "object": "object",
                                "privacy_opt_out": True,
                                "regulation_o": True,
                                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                            },
                            "business_description": "business_description",
                            "business_name": "business_name",
                            "citizenship_country": "citizenship_country",
                            "compliance_details": {},
                            "connection_id": "connection_id",
                            "country_of_incorporation": "country_of_incorporation",
                            "date_formed": parse_date("2019-12-27"),
                            "date_of_birth": parse_date("2019-12-27"),
                            "documents": [
                                {
                                    "document_type": "articles_of_incorporation",
                                    "file_data": "file_data",
                                    "filename": "filename",
                                }
                            ],
                            "doing_business_as_names": ["string"],
                            "email": "email",
                            "expected_activity_volume": 0,
                            "external_id": "external_id",
                            "first_name": "first_name",
                            "identifications": [
                                {
                                    "id_number": "id_number",
                                    "id_type": "ar_cuil",
                                    "documents": [
                                        {
                                            "document_type": "articles_of_incorporation",
                                            "file_data": "file_data",
                                            "filename": "filename",
                                        }
                                    ],
                                    "expiration_date": parse_date("2019-12-27"),
                                    "issuing_country": "issuing_country",
                                    "issuing_region": "issuing_region",
                                }
                            ],
                            "industry_classifications": [
                                {
                                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                    "classification_codes": ["string"],
                                    "classification_type": "anzsic",
                                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "live_mode": True,
                                    "object": "object",
                                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                }
                            ],
                            "intended_use": "intended_use",
                            "last_name": "last_name",
                            "legal_entity_associations": [],
                            "legal_entity_type": "business",
                            "legal_structure": "corporation",
                            "listed_exchange": "listed_exchange",
                            "metadata": {
                                "key": "value",
                                "foo": "bar",
                                "modern": "treasury",
                            },
                            "middle_name": "middle_name",
                            "operating_jurisdictions": ["string"],
                            "phone_numbers": [{"phone_number": "phone_number"}],
                            "politically_exposed_person": True,
                            "preferred_name": "preferred_name",
                            "prefix": "prefix",
                            "primary_social_media_sites": ["string"],
                            "regulators": [
                                {
                                    "jurisdiction": "jurisdiction",
                                    "name": "name",
                                    "registration_number": "registration_number",
                                }
                            ],
                            "risk_rating": "low",
                            "service_provider_legal_entity_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "suffix": "suffix",
                            "third_party_verification": {
                                "outcome": "passed",
                                "vendor": "persona",
                                "vendor_verification_id": "vendor_verification_id",
                                "verification_category": "legal_name",
                                "verification_method": "verification_method",
                                "verification_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "comment": "comment",
                            },
                            "third_party_verifications": [
                                {
                                    "outcome": "passed",
                                    "vendor": "persona",
                                    "vendor_verification_id": "vendor_verification_id",
                                    "verification_category": "legal_name",
                                    "verification_method": "verification_method",
                                    "verification_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "comment": "comment",
                                }
                            ],
                            "ticker_symbol": "ticker_symbol",
                            "wealth_and_employment_details": {
                                "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "annual_income": 0,
                                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "employer_country": "employer_country",
                                "employer_name": "employer_name",
                                "employer_state": "employer_state",
                                "employment_status": "employed",
                                "income_country": "income_country",
                                "income_source": "family_support",
                                "income_state": "income_state",
                                "industry": "accounting",
                                "live_mode": True,
                                "object": "object",
                                "occupation": "consulting",
                                "source_of_funds": "alimony",
                                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "wealth_source": "business_sale",
                            },
                            "website": "website",
                        },
                        "child_legal_entity_id": "child_legal_entity_id",
                        "ownership_percentage": 0,
                        "title": "title",
                    }
                ],
                "legal_structure": "corporation",
                "listed_exchange": "listed_exchange",
                "metadata": {
                    "key": "value",
                    "foo": "bar",
                    "modern": "treasury",
                },
                "middle_name": "middle_name",
                "operating_jurisdictions": ["string"],
                "phone_numbers": [{"phone_number": "phone_number"}],
                "politically_exposed_person": True,
                "preferred_name": "preferred_name",
                "prefix": "prefix",
                "primary_social_media_sites": ["string"],
                "regulators": [
                    {
                        "jurisdiction": "jurisdiction",
                        "name": "name",
                        "registration_number": "registration_number",
                    }
                ],
                "risk_rating": "low",
                "service_provider_legal_entity_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "suffix": "suffix",
                "third_party_verification": {
                    "outcome": "passed",
                    "vendor": "persona",
                    "vendor_verification_id": "vendor_verification_id",
                    "verification_category": "legal_name",
                    "verification_method": "verification_method",
                    "verification_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "comment": "comment",
                },
                "third_party_verifications": [
                    {
                        "outcome": "passed",
                        "vendor": "persona",
                        "vendor_verification_id": "vendor_verification_id",
                        "verification_category": "legal_name",
                        "verification_method": "verification_method",
                        "verification_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "comment": "comment",
                    }
                ],
                "ticker_symbol": "ticker_symbol",
                "wealth_and_employment_details": {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "annual_income": 0,
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "employer_country": "employer_country",
                    "employer_name": "employer_name",
                    "employer_state": "employer_state",
                    "employment_status": "employed",
                    "income_country": "income_country",
                    "income_source": "family_support",
                    "income_state": "income_state",
                    "industry": "accounting",
                    "live_mode": True,
                    "object": "object",
                    "occupation": "consulting",
                    "source_of_funds": "alimony",
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "wealth_source": "business_sale",
                },
                "website": "website",
            },
            legal_entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
            send_remittance_advice=True,
            taxpayer_identifier="taxpayer_identifier",
            verification_status="verification_status",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.counterparties.with_raw_response.create(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.counterparties.with_streaming_response.create(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = await response.parse()
            assert_matches_type(Counterparty, counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.retrieve(
            "id",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.counterparties.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.counterparties.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = await response.parse()
            assert_matches_type(Counterparty, counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.counterparties.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.update(
            id="id",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.update(
            id="id",
            email="dev@stainless.com",
            external_id="external_id",
            legal_entity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={"foo": "string"},
            name="name",
            send_remittance_advice=True,
            taxpayer_identifier="taxpayer_identifier",
        )
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.counterparties.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(Counterparty, counterparty, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.counterparties.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = await response.parse()
            assert_matches_type(Counterparty, counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.counterparties.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.list()
        assert_matches_type(AsyncPage[Counterparty], counterparty, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.list(
            after_cursor="after_cursor",
            created_at_lower_bound=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_upper_bound=parse_datetime("2019-12-27T18:11:19.117Z"),
            email="dev@stainless.com",
            external_id="external_id",
            legal_entity_id="legal_entity_id",
            metadata={"foo": "string"},
            name="name",
            per_page=0,
        )
        assert_matches_type(AsyncPage[Counterparty], counterparty, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.counterparties.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(AsyncPage[Counterparty], counterparty, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.counterparties.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = await response.parse()
            assert_matches_type(AsyncPage[Counterparty], counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.delete(
            "id",
        )
        assert counterparty is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.counterparties.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert counterparty is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.counterparties.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = await response.parse()
            assert counterparty is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.counterparties.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_collect_account(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.collect_account(
            id="id",
            direction="credit",
        )
        assert_matches_type(CounterpartyCollectAccountResponse, counterparty, path=["response"])

    @parametrize
    async def test_method_collect_account_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        counterparty = await async_client.counterparties.collect_account(
            id="id",
            direction="credit",
            custom_redirect="https://example.com",
            fields=["name"],
            send_email=True,
        )
        assert_matches_type(CounterpartyCollectAccountResponse, counterparty, path=["response"])

    @parametrize
    async def test_raw_response_collect_account(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.counterparties.with_raw_response.collect_account(
            id="id",
            direction="credit",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        counterparty = response.parse()
        assert_matches_type(CounterpartyCollectAccountResponse, counterparty, path=["response"])

    @parametrize
    async def test_streaming_response_collect_account(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.counterparties.with_streaming_response.collect_account(
            id="id",
            direction="credit",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            counterparty = await response.parse()
            assert_matches_type(CounterpartyCollectAccountResponse, counterparty, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_collect_account(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.counterparties.with_raw_response.collect_account(
                id="",
                direction="credit",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["HoldRetrieveResponse"]


class HoldRetrieveResponse(BaseModel):
    id: str

    created_at: datetime

    object: Literal["hold"]
    """The type of object"""

    status: Literal["active", "resolved"]
    """The status of the hold"""

    target_id: str
    """The ID of the target being held"""

    target_type: Literal["payment_order"]
    """The type of target being held"""

    updated_at: datetime

    live_mode: Optional[bool] = None
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Optional[Dict[str, str]] = None
    """Additional metadata for the hold"""

    reason: Optional[str] = None
    """The reason for the hold"""

    resolution: Optional[str] = None
    """The resolution of the hold"""

    resolved_at: Optional[datetime] = None
    """When the hold was resolved"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import (
    InternalAccount,
    InternalAccountUpdateAccountCapabilityResponse,
)
from modern_treasury._utils import parse_datetime
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestInternalAccounts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        internal_account = client.internal_accounts.create(
            connection_id="connection_id",
            currency="USD",
            name="name",
            party_name="party_name",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        internal_account = client.internal_accounts.create(
            connection_id="connection_id",
            currency="USD",
            name="name",
            party_name="party_name",
            account_capabilities=[
                {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "direction": "credit",
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "identifier": "identifier",
                    "live_mode": True,
                    "object": "object",
                    "payment_type": "ach",
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            account_type="checking",
            counterparty_id="counterparty_id",
            external_id="external_id",
            legal_entity_id="legal_entity_id",
            metadata={"foo": "string"},
            parent_account_id="parent_account_id",
            party_address={
                "country": "country",
                "line1": "line1",
                "locality": "locality",
                "postal_code": "postal_code",
                "region": "region",
                "line2": "line2",
            },
            vendor_attributes={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.internal_accounts.with_raw_response.create(
            connection_id="connection_id",
            currency="USD",
            name="name",
            party_name="party_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.internal_accounts.with_streaming_response.create(
            connection_id="connection_id",
            currency="USD",
            name="name",
            party_name="party_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = response.parse()
            assert_matches_type(InternalAccount, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        internal_account = client.internal_accounts.retrieve(
            "id",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.internal_accounts.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.internal_accounts.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = response.parse()
            assert_matches_type(InternalAccount, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.internal_accounts.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: ModernTreasury) -> None:
        internal_account = client.internal_accounts.update(
            id="id",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: ModernTreasury) -> None:
        internal_account = client.internal_accounts.update(
            id="id",
            contra_ledger_account_id="contra_ledger_account_id",
            counterparty_id="counterparty_id",
            external_id="external_id",
            ledger_account_id="ledger_account_id",
            metadata={"foo": "string"},
            name="name",
            parent_account_id="parent_account_id",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: ModernTreasury) -> None:
        response = client.internal_accounts.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: ModernTreasury) -> None:
        with client.internal_accounts.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = response.parse()
            assert_matches_type(InternalAccount, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.internal_accounts.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        internal_account = client.internal_accounts.list()
        assert_matches_type(SyncPage[InternalAccount], internal_account, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        internal_account = client.internal_accounts.list(
            after_cursor="after_cursor",
            counterparty_id="counterparty_id",
            currency="AED",
            external_id="external_id",
            legal_entity_id="legal_entity_id",
            metadata={"foo": "string"},
            payment_direction="credit",
            payment_type="ach",
            per_page=0,
            status="active",
        )
        assert_matches_type(SyncPage[InternalAccount], internal_account, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.internal_accounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(SyncPage[InternalAccount], internal_account, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.internal_accounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = response.parse()
            assert_matches_type(SyncPage[InternalAccount], internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_request_closure(self, client: ModernTreasury) -> None:
        internal_account = client.internal_accounts.request_closure(
            "id",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_raw_response_request_closure(self, client: ModernTreasury) -> None:
        response = client.internal_accounts.with_raw_response.request_closure(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    def test_streaming_response_request_closure(self, client: ModernTreasury) -> None:
        with client.internal_accounts.with_streaming_response.request_closure(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = response.parse()
            assert_matches_type(InternalAccount, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_request_closure(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.internal_accounts.with_raw_response.request_closure(
                "",
            )

    @parametrize
    def test_method_update_account_capability(self, client: ModernTreasury) -> None:
        internal_account = client.internal_accounts.update_account_capability(
            id="id",
            internal_account_id="internal_account_id",
            identifier="identifier",
        )
        assert_matches_type(InternalAccountUpdateAccountCapabilityResponse, internal_account, path=["response"])

    @parametrize
    def test_raw_response_update_account_capability(self, client: ModernTreasury) -> None:
        response = client.internal_accounts.with_raw_response.update_account_capability(
            id="id",
            internal_account_id="internal_account_id",
            identifier="identifier",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccountUpdateAccountCapabilityResponse, internal_account, path=["response"])

    @parametrize
    def test_streaming_response_update_account_capability(self, client: ModernTreasury) -> None:
        with client.internal_accounts.with_streaming_response.update_account_capability(
            id="id",
            internal_account_id="internal_account_id",
            identifier="identifier",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = response.parse()
            assert_matches_type(InternalAccountUpdateAccountCapabilityResponse, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_account_capability(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `internal_account_id` but received ''"):
            client.internal_accounts.with_raw_response.update_account_capability(
                id="id",
                internal_account_id="",
                identifier="identifier",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.internal_accounts.with_raw_response.update_account_capability(
                id="",
                internal_account_id="internal_account_id",
                identifier="identifier",
            )


class TestAsyncInternalAccounts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        internal_account = await async_client.internal_accounts.create(
            connection_id="connection_id",
            currency="USD",
            name="name",
            party_name="party_name",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        internal_account = await async_client.internal_accounts.create(
            connection_id="connection_id",
            currency="USD",
            name="name",
            party_name="party_name",
            account_capabilities=[
                {
                    "id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "direction": "credit",
                    "discarded_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "identifier": "identifier",
                    "live_mode": True,
                    "object": "object",
                    "payment_type": "ach",
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            account_type="checking",
            counterparty_id="counterparty_id",
            external_id="external_id",
            legal_entity_id="legal_entity_id",
            metadata={"foo": "string"},
            parent_account_id="parent_account_id",
            party_address={
                "country": "country",
                "line1": "line1",
                "locality": "locality",
                "postal_code": "postal_code",
                "region": "region",
                "line2": "line2",
            },
            vendor_attributes={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.internal_accounts.with_raw_response.create(
            connection_id="connection_id",
            currency="USD",
            name="name",
            party_name="party_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.internal_accounts.with_streaming_response.create(
            connection_id="connection_id",
            currency="USD",
            name="name",
            party_name="party_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = await response.parse()
            assert_matches_type(InternalAccount, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        internal_account = await async_client.internal_accounts.retrieve(
            "id",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.internal_accounts.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.internal_accounts.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = await response.parse()
            assert_matches_type(InternalAccount, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.internal_accounts.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncModernTreasury) -> None:
        internal_account = await async_client.internal_accounts.update(
            id="id",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        internal_account = await async_client.internal_accounts.update(
            id="id",
            contra_ledger_account_id="contra_ledger_account_id",
            counterparty_id="counterparty_id",
            external_id="external_id",
            ledger_account_id="ledger_account_id",
            metadata={"foo": "string"},
            name="name",
            parent_account_id="parent_account_id",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.internal_accounts.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.internal_accounts.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = await response.parse()
            assert_matches_type(InternalAccount, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.internal_accounts.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        internal_account = await async_client.internal_accounts.list()
        assert_matches_type(AsyncPage[InternalAccount], internal_account, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        internal_account = await async_client.internal_accounts.list(
            after_cursor="after_cursor",
            counterparty_id="counterparty_id",
            currency="AED",
            external_id="external_id",
            legal_entity_id="legal_entity_id",
            metadata={"foo": "string"},
            payment_direction="credit",
            payment_type="ach",
            per_page=0,
            status="active",
        )
        assert_matches_type(AsyncPage[InternalAccount], internal_account, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.internal_accounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(AsyncPage[InternalAccount], internal_account, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.internal_accounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = await response.parse()
            assert_matches_type(AsyncPage[InternalAccount], internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_request_closure(self, async_client: AsyncModernTreasury) -> None:
        internal_account = await async_client.internal_accounts.request_closure(
            "id",
        )
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_raw_response_request_closure(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.internal_accounts.with_raw_response.request_closure(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccount, internal_account, path=["response"])

    @parametrize
    async def test_streaming_response_request_closure(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.internal_accounts.with_streaming_response.request_closure(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = await response.parse()
            assert_matches_type(InternalAccount, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_request_closure(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.internal_accounts.with_raw_response.request_closure(
                "",
            )

    @parametrize
    async def test_method_update_account_capability(self, async_client: AsyncModernTreasury) -> None:
        internal_account = await async_client.internal_accounts.update_account_capability(
            id="id",
            internal_account_id="internal_account_id",
            identifier="identifier",
        )
        assert_matches_type(InternalAccountUpdateAccountCapabilityResponse, internal_account, path=["response"])

    @parametrize
    async def test_raw_response_update_account_capability(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.internal_accounts.with_raw_response.update_account_capability(
            id="id",
            internal_account_id="internal_account_id",
            identifier="identifier",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        internal_account = response.parse()
        assert_matches_type(InternalAccountUpdateAccountCapabilityResponse, internal_account, path=["response"])

    @parametrize
    async def test_streaming_response_update_account_capability(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.internal_accounts.with_streaming_response.update_account_capability(
            id="id",
            internal_account_id="internal_account_id",
            identifier="identifier",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            internal_account = await response.parse()
            assert_matches_type(InternalAccountUpdateAccountCapabilityResponse, internal_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_account_capability(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `internal_account_id` but received ''"):
            await async_client.internal_accounts.with_raw_response.update_account_capability(
                id="id",
                internal_account_id="",
                identifier="identifier",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.internal_accounts.with_raw_response.update_account_capability(
                id="",
                internal_account_id="internal_account_id",
                identifier="identifier",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ContactDetail"]


class ContactDetail(BaseModel):
    id: str

    contact_identifier: str

    contact_identifier_type: Literal["email", "phone_number", "website"]

    created_at: datetime

    discarded_at: Optional[datetime] = None

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    updated_at: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from .external_account_type import ExternalAccountType
from .shared_params.address_request import AddressRequest
from .contact_detail_create_request_param import ContactDetailCreateRequestParam
from .shared_params.ledger_account_create_request import LedgerAccountCreateRequest

__all__ = ["ExternalAccountCreateParams", "AccountDetails", "AccountDetail", "RoutingDetails", "RoutingDetail"]


class ExternalAccountCreateParams(TypedDict, total=False):
    counterparty_id: Required[Optional[str]]

    account_details: Iterable[AccountDetail]

    account_type: ExternalAccountType
    """Can be `checking`, `savings` or `other`."""

    contact_details: Iterable[ContactDetailCreateRequestParam]

    external_id: Optional[str]
    """An optional user-defined 180 character unique identifier."""

    ledger_account: LedgerAccountCreateRequest
    """Specifies a ledger account object that will be created with the external
    account.

    The resulting ledger account is linked to the external account for
    auto-ledgering Payment objects. See
    https://docs.moderntreasury.com/docs/linking-to-other-modern-treasury-objects
    for more details.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    name: Optional[str]
    """A nickname for the external account.

    This is only for internal usage and won't affect any payments
    """

    party_address: AddressRequest
    """Required if receiving wire payments."""

    party_identifier: str

    party_name: str
    """
    If this value isn't provided, it will be inherited from the counterparty's name.
    """

    party_type: Optional[Literal["business", "individual"]]
    """Either `individual` or `business`."""

    plaid_processor_token: str
    """
    If you've enabled the Modern Treasury + Plaid integration in your Plaid account,
    you can pass the processor token in this field.
    """

    routing_details: Iterable[RoutingDetail]


class AccountDetail(TypedDict, total=False):
    account_number: Required[str]

    account_number_type: Literal[
        "au_number",
        "base_address",
        "card_token",
        "clabe",
        "ethereum_address",
        "hk_number",
        "iban",
        "id_number",
        "nz_number",
        "other",
        "pan",
        "polygon_address",
        "sg_number",
        "solana_address",
        "wallet_address",
    ]


AccountDetails = AccountDetail
"""This type is deprecated and will be removed in a future release.

Please use AccountDetail instead.
"""


class RoutingDetail(TypedDict, total=False):
    routing_number: Required[str]

    routing_number_type: Required[
        Literal[
            "aba",
            "au_bsb",
            "br_codigo",
            "ca_cpa",
            "chips",
            "cnaps",
            "dk_interbank_clearing_code",
            "gb_sort_code",
            "hk_interbank_clearing_code",
            "hu_interbank_clearing_code",
            "id_sknbi_code",
            "il_bank_code",
            "in_ifsc",
            "jp_zengin_code",
            "my_branch_code",
            "mx_bank_identifier",
            "nz_national_clearing_code",
            "pl_national_clearing_code",
            "se_bankgiro_clearing_code",
            "sg_interbank_clearing_code",
            "swift",
            "za_national_clearing_code",
        ]
    ]

    payment_type: Literal[
        "ach",
        "au_becs",
        "bacs",
        "base",
        "book",
        "card",
        "chats",
        "check",
        "cross_border",
        "dk_nets",
        "eft",
        "ethereum",
        "gb_fps",
        "hu_ics",
        "interac",
        "masav",
        "mx_ccen",
        "neft",
        "nics",
        "nz_becs",
        "pl_elixir",
        "polygon",
        "provxchange",
        "ro_sent",
        "rtp",
        "se_bankgirot",
        "sen",
        "sepa",
        "sg_giro",
        "sic",
        "signet",
        "sknbi",
        "solana",
        "stablecoin",
        "wire",
        "zengin",
    ]


RoutingDetails = RoutingDetail
"""This type is deprecated and will be removed in a future release.

Please use RoutingDetail instead.
"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from . import (
    shared,
    invoice,
    bulk_result,
    legal_entity,
    payment_order,
    return_object,
    child_legal_entity,
    legal_entity_association,
)
from .. import _compat
from .event import Event as Event
from .ledger import Ledger as Ledger
from .shared import (
    Address as Address,
    Currency as Currency,
    AccountsType as AccountsType,
    AsyncResponse as AsyncResponse,
    ContactDetail as ContactDetail,
    LedgerBalance as LedgerBalance,
    AddressRequest as AddressRequest,
    LedgerBalances as LedgerBalances,
    ForeignExchangeRate as ForeignExchangeRate,
    TransactionDirection as TransactionDirection,
    ChildLegalEntityCreate as ChildLegalEntityCreate,
    ThirdPartyVerification as ThirdPartyVerification,
    LedgerEntryCreateRequest as LedgerEntryCreateRequest,
    LedgerAccountCreateRequest as LedgerAccountCreateRequest,
    IdentificationCreateRequest as IdentificationCreateRequest,
    LedgerTransactionCreateRequest as LedgerTransactionCreateRequest,
    LegalEntityAddressCreateRequest as LegalEntityAddressCreateRequest,
    LegalEntityIndustryClassification as LegalEntityIndustryClassification,
    LegalEntityAssociationInlineCreate as LegalEntityAssociationInlineCreate,
)
from .invoice import Invoice as Invoice
from .document import Document as Document
from .line_item import LineItem as LineItem
from .connection import Connection as Connection
from .bulk_result import BulkResult as BulkResult
from .transaction import Transaction as Transaction
from .bulk_request import BulkRequest as BulkRequest
from .counterparty import Counterparty as Counterparty
from .ledger_entry import LedgerEntry as LedgerEntry
from .legal_entity import LegalEntity as LegalEntity
from .payment_flow import PaymentFlow as PaymentFlow
from .payment_order import PaymentOrder as PaymentOrder
from .ping_response import PingResponse as PingResponse
from .return_object import ReturnObject as ReturnObject
from .account_detail import AccountDetail as AccountDetail
from .ledger_account import LedgerAccount as LedgerAccount
from .routing_detail import RoutingDetail as RoutingDetail
from .virtual_account import VirtualAccount as VirtualAccount
from .expected_payment import ExpectedPayment as ExpectedPayment
from .external_account import ExternalAccount as ExternalAccount
from .hold_list_params import HoldListParams as HoldListParams
from .internal_account import InternalAccount as InternalAccount
from .event_list_params import EventListParams as EventListParams
from .payment_reference import PaymentReference as PaymentReference
from .child_legal_entity import ChildLegalEntity as ChildLegalEntity
from .hold_create_params import HoldCreateParams as HoldCreateParams
from .hold_list_response import HoldListResponse as HoldListResponse
from .hold_update_params import HoldUpdateParams as HoldUpdateParams
from .ledger_list_params import LedgerListParams as LedgerListParams
from .ledger_transaction import LedgerTransaction as LedgerTransaction
from .payment_order_type import PaymentOrderType as PaymentOrderType
from .return_list_params import ReturnListParams as ReturnListParams
from .invoice_list_params import InvoiceListParams as InvoiceListParams
from .reconciliation_rule import ReconciliationRule as ReconciliationRule
from .document_list_params import DocumentListParams as DocumentListParams
from .hold_create_response import HoldCreateResponse as HoldCreateResponse
from .hold_update_response import HoldUpdateResponse as HoldUpdateResponse
from .ledger_create_params import LedgerCreateParams as LedgerCreateParams
from .ledger_update_params import LedgerUpdateParams as LedgerUpdateParams
from .return_create_params import ReturnCreateParams as ReturnCreateParams
from .expected_payment_type import ExpectedPaymentType as ExpectedPaymentType
from .external_account_type import ExternalAccountType as ExternalAccountType
from .invoice_create_params import InvoiceCreateParams as InvoiceCreateParams
from .invoice_update_params import InvoiceUpdateParams as InvoiceUpdateParams
from .line_item_list_params import LineItemListParams as LineItemListParams
from .payment_order_subtype import PaymentOrderSubtype as PaymentOrderSubtype
from .connection_list_params import ConnectionListParams as ConnectionListParams
from .document_create_params import DocumentCreateParams as DocumentCreateParams
from .foreign_exchange_quote import ForeignExchangeQuote as ForeignExchangeQuote
from .hold_retrieve_response import HoldRetrieveResponse as HoldRetrieveResponse
from .account_collection_flow import AccountCollectionFlow as AccountCollectionFlow
from .bulk_result_list_params import BulkResultListParams as BulkResultListParams
from .connection_legal_entity import ConnectionLegalEntity as ConnectionLegalEntity
from .incoming_payment_detail import IncomingPaymentDetail as IncomingPaymentDetail
from .ledger_account_category import LedgerAccountCategory as LedgerAccountCategory
from .line_item_update_params import LineItemUpdateParams as LineItemUpdateParams
from .transaction_list_params import TransactionListParams as TransactionListParams
from .bulk_request_list_params import BulkRequestListParams as BulkRequestListParams
from .counterparty_list_params import CounterpartyListParams as CounterpartyListParams
from .ledger_entry_list_params import LedgerEntryListParams as LedgerEntryListParams
from .legal_entity_association import LegalEntityAssociation as LegalEntityAssociation
from .legal_entity_list_params import LegalEntityListParams as LegalEntityListParams
from .payment_flow_list_params import PaymentFlowListParams as PaymentFlowListParams
from .journal_entry_list_params import JournalEntryListParams as JournalEntryListParams
from .ledger_account_settlement import LedgerAccountSettlement as LedgerAccountSettlement
from .payment_order_list_params import PaymentOrderListParams as PaymentOrderListParams
from .reconciliation_rule_param import ReconciliationRuleParam as ReconciliationRuleParam
from .transaction_create_params import TransactionCreateParams as TransactionCreateParams
from .transaction_update_params import TransactionUpdateParams as TransactionUpdateParams
from .account_detail_list_params import AccountDetailListParams as AccountDetailListParams
from .bulk_request_create_params import BulkRequestCreateParams as BulkRequestCreateParams
from .counterparty_create_params import CounterpartyCreateParams as CounterpartyCreateParams
from .counterparty_update_params import CounterpartyUpdateParams as CounterpartyUpdateParams
from .journal_report_list_params import JournalReportListParams as JournalReportListParams
from .ledger_account_list_params import LedgerAccountListParams as LedgerAccountListParams
from .ledger_entry_update_params import LedgerEntryUpdateParams as LedgerEntryUpdateParams
from .legal_entity_create_params import LegalEntityCreateParams as LegalEntityCreateParams
from .legal_entity_update_params import LegalEntityUpdateParams as LegalEntityUpdateParams
from .payment_action_list_params import PaymentActionListParams as PaymentActionListParams
from .payment_flow_create_params import PaymentFlowCreateParams as PaymentFlowCreateParams
from .payment_flow_update_params import PaymentFlowUpdateParams as PaymentFlowUpdateParams
from .routing_detail_list_params import RoutingDetailListParams as RoutingDetailListParams
from .payment_order_create_params import PaymentOrderCreateParams as PaymentOrderCreateParams
from .payment_order_update_params import PaymentOrderUpdateParams as PaymentOrderUpdateParams
from .virtual_account_list_params import VirtualAccountListParams as VirtualAccountListParams
from .account_detail_create_params import AccountDetailCreateParams as AccountDetailCreateParams
from .expected_payment_list_params import ExpectedPaymentListParams as ExpectedPaymentListParams
from .external_account_list_params import ExternalAccountListParams as ExternalAccountListParams
from .internal_account_list_params import InternalAccountListParams as InternalAccountListParams
from .journal_report_update_params import JournalReportUpdateParams as JournalReportUpdateParams
from .ledger_account_create_params import LedgerAccountCreateParams as LedgerAccountCreateParams
from .ledger_account_update_params import LedgerAccountUpdateParams as LedgerAccountUpdateParams
from .ledger_entry_retrieve_params import LedgerEntryRetrieveParams as LedgerEntryRetrieveParams
from .payment_action_create_params import PaymentActionCreateParams as PaymentActionCreateParams
from .payment_action_list_response import PaymentActionListResponse as PaymentActionListResponse
from .payment_action_update_params import PaymentActionUpdateParams as PaymentActionUpdateParams
from .routing_detail_create_params import RoutingDetailCreateParams as RoutingDetailCreateParams
from .payment_reference_list_params import PaymentReferenceListParams as PaymentReferenceListParams
from .routing_number_lookup_request import RoutingNumberLookupRequest as RoutingNumberLookupRequest
from .virtual_account_create_params import VirtualAccountCreateParams as VirtualAccountCreateParams
from .virtual_account_update_params import VirtualAccountUpdateParams as VirtualAccountUpdateParams
from .expected_payment_create_params import ExpectedPaymentCreateParams as ExpectedPaymentCreateParams
from .expected_payment_update_params import ExpectedPaymentUpdateParams as ExpectedPaymentUpdateParams
from .external_account_create_params import ExternalAccountCreateParams as ExternalAccountCreateParams
from .external_account_update_params import ExternalAccountUpdateParams as ExternalAccountUpdateParams
from .external_account_verify_params import ExternalAccountVerifyParams as ExternalAccountVerifyParams
from .internal_account_create_params import InternalAccountCreateParams as InternalAccountCreateParams
from .internal_account_update_params import InternalAccountUpdateParams as InternalAccountUpdateParams
from .ledger_account_balance_monitor import LedgerAccountBalanceMonitor as LedgerAccountBalanceMonitor
from .ledger_account_retrieve_params import LedgerAccountRetrieveParams as LedgerAccountRetrieveParams
from .ledger_transaction_list_params import LedgerTransactionListParams as LedgerTransactionListParams
from .payment_action_create_response import PaymentActionCreateResponse as PaymentActionCreateResponse
from .payment_action_update_response import PaymentActionUpdateResponse as PaymentActionUpdateResponse
from .external_account_verify_response import ExternalAccountVerifyResponse as ExternalAccountVerifyResponse
from .ledger_transaction_create_params import LedgerTransactionCreateParams as LedgerTransactionCreateParams
from .ledger_transaction_update_params import LedgerTransactionUpdateParams as LedgerTransactionUpdateParams
from .payment_action_retrieve_response import PaymentActionRetrieveResponse as PaymentActionRetrieveResponse
from .payment_order_create_async_params import PaymentOrderCreateAsyncParams as PaymentOrderCreateAsyncParams
from .foreign_exchange_quote_list_params import ForeignExchangeQuoteListParams as ForeignExchangeQuoteListParams
from .account_collection_flow_list_params import AccountCollectionFlowListParams as AccountCollectionFlowListParams
from .connection_legal_entity_list_params import ConnectionLegalEntityListParams as ConnectionLegalEntityListParams
from .contact_detail_create_request_param import ContactDetailCreateRequestParam as ContactDetailCreateRequestParam
from .counterparty_collect_account_params import CounterpartyCollectAccountParams as CounterpartyCollectAccountParams
from .incoming_payment_detail_list_params import IncomingPaymentDetailListParams as IncomingPaymentDetailListParams
from .ledger_account_category_list_params import LedgerAccountCategoryListParams as LedgerAccountCategoryListParams
from .foreign_exchange_quote_create_params import ForeignExchangeQuoteCreateParams as ForeignExchangeQuoteCreateParams
from .account_collection_flow_create_params import (
    AccountCollectionFlowCreateParams as AccountCollectionFlowCreateParams,
)
from .account_collection_flow_update_params import (
    AccountCollectionFlowUpdateParams as AccountCollectionFlowUpdateParams,
)
from .connection_legal_entity_create_params import (
    ConnectionLegalEntityCreateParams as ConnectionLegalEntityCreateParams,
)
from .connection_legal_entity_update_params import (
    ConnectionLegalEntityUpdateParams as ConnectionLegalEntityUpdateParams,
)
from .counterparty_collect_account_response import (
    CounterpartyCollectAccountResponse as CounterpartyCollectAccountResponse,
)
from .incoming_payment_detail_update_params import (
    IncomingPaymentDetailUpdateParams as IncomingPaymentDetailUpdateParams,
)
from .ledger_account_category_create_params import (
    LedgerAccountCategoryCreateParams as LedgerAccountCategoryCreateParams,
)
from .ledger_account_category_update_params import (
    LedgerAccountCategoryUpdateParams as LedgerAccountCategoryUpdateParams,
)
from .ledger_account_settlement_list_params import (
    LedgerAccountSettlementListParams as LedgerAccountSettlementListParams,
)
from .ledger_account_statement_create_params import (
    LedgerAccountStatementCreateParams as LedgerAccountStatementCreateParams,
)
from .ledger_account_category_retrieve_params import (
    LedgerAccountCategoryRetrieveParams as LedgerAccountCategoryRetrieveParams,
)
from .ledger_account_settlement_create_params import (
    LedgerAccountSettlementCreateParams as LedgerAccountSettlementCreateParams,
)
from .ledger_account_settlement_update_params import (
    LedgerAccountSettlementUpdateParams as LedgerAccountSettlementUpdateParams,
)
from .ledger_account_statement_create_response import (
    LedgerAccountStatementCreateResponse as LedgerAccountStatementCreateResponse,
)
from .ledger_transaction_create_reversal_params import (
    LedgerTransactionCreateReversalParams as LedgerTransactionCreateReversalParams,
)
from .validation_validate_routing_number_params import (
    ValidationValidateRoutingNumberParams as ValidationValidateRoutingNumberParams,
)
from .ledger_account_balance_monitor_list_params import (
    LedgerAccountBalanceMonitorListParams as LedgerAccountBalanceMonitorListParams,
)
from .ledger_account_statement_retrieve_response import (
    LedgerAccountStatementRetrieveResponse as LedgerAccountStatementRetrieveResponse,
)
from .incoming_payment_detail_create_async_params import (
    IncomingPaymentDetailCreateAsyncParams as IncomingPaymentDetailCreateAsyncParams,
)
from .ledger_account_balance_monitor_create_params import (
    LedgerAccountBalanceMonitorCreateParams as LedgerAccountBalanceMonitorCreateParams,
)
from .ledger_account_balance_monitor_update_params import (
    LedgerAccountBalanceMonitorUpdateParams as LedgerAccountBalanceMonitorUpdateParams,
)
from .external_account_complete_verification_params import (
    ExternalAccountCompleteVerificationParams as ExternalAccountCompleteVerificationParams,
)
from .ledger_transaction_create_partial_post_params import (
    LedgerTransactionCreatePartialPostParams as LedgerTransactionCreatePartialPostParams,
)
from .internal_account_update_account_capability_params import (
    InternalAccountUpdateAccountCapabilityParams as InternalAccountUpdateAccountCapabilityParams,
)
from .internal_account_update_account_capability_response import (
    InternalAccountUpdateAccountCapabilityResponse as InternalAccountUpdateAccountCapabilityResponse,
)

# Rebuild cyclical models only after all modules are imported.
# This ensures that, when building the deferred (due to cyclical references) model schema,
# Pydantic can resolve the necessary references.
# See: https://github.com/pydantic/pydantic/issues/11250 for more context.
if _compat.PYDANTIC_V1:
    invoice.Invoice.update_forward_refs()  # type: ignore
    payment_order.PaymentOrder.update_forward_refs()  # type: ignore
    return_object.ReturnObject.update_forward_refs()  # type: ignore
    bulk_result.BulkResult.update_forward_refs()  # type: ignore
    legal_entity.LegalEntity.update_forward_refs()  # type: ignore
    child_legal_entity.ChildLegalEntity.update_forward_refs()  # type: ignore
    legal_entity_association.LegalEntityAssociation.update_forward_refs()  # type: ignore
    shared.child_legal_entity_create.ChildLegalEntityCreate.update_forward_refs()  # type: ignore
    shared.legal_entity_association_inline_create.LegalEntityAssociationInlineCreate.update_forward_refs()  # type: ignore
else:
    invoice.Invoice.model_rebuild(_parent_namespace_depth=0)
    payment_order.PaymentOrder.model_rebuild(_parent_namespace_depth=0)
    return_object.ReturnObject.model_rebuild(_parent_namespace_depth=0)
    bulk_result.BulkResult.model_rebuild(_parent_namespace_depth=0)
    legal_entity.LegalEntity.model_rebuild(_parent_namespace_depth=0)
    child_legal_entity.ChildLegalEntity.model_rebuild(_parent_namespace_depth=0)
    legal_entity_association.LegalEntityAssociation.model_rebuild(_parent_namespace_depth=0)
    shared.child_legal_entity_create.ChildLegalEntityCreate.model_rebuild(_parent_namespace_depth=0)
    shared.legal_entity_association_inline_create.LegalEntityAssociationInlineCreate.model_rebuild(
        _parent_namespace_depth=0
    )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BalanceReportCreateParams", "Balances", "Balance"]


class BalanceReportCreateParams(TypedDict, total=False):
    as_of_date: Required[Annotated[Union[str, date], PropertyInfo(format="iso8601")]]
    """The date of the balance report in local time."""

    as_of_time: Required[str]
    """The time (24-hour clock) of the balance report in local time."""

    balance_report_type: Required[Literal["intraday", "other", "previous_day", "real_time"]]
    """The specific type of balance report.

    One of `intraday`, `previous_day`, `real_time`, or `other`.
    """

    balances: Required[Iterable[Balance]]
    """An array of `Balance` objects."""


class Balance(TypedDict, total=False):
    amount: Required[int]
    """The balance amount."""

    balance_type: Required[
        Literal[
            "closing_available",
            "closing_ledger",
            "current_available",
            "current_ledger",
            "opening_available",
            "opening_available_next_business_day",
            "opening_ledger",
            "other",
            "previously_closed_book",
        ]
    ]
    """The specific type of balance reported.

    One of `opening_ledger`, `closing_ledger`, `current_ledger`,
    `opening_available`, `opening_available_next_business_day`, `closing_available`,
    `current_available`, 'previously_closed_book', or `other`.
    """

    vendor_code: Required[str]
    """The code used by the bank when reporting this specific balance."""

    vendor_code_type: Required[Optional[str]]
    """The type of `vendor_code` being reported.

    Can be one of `bai2`, `bankprov`, `bnk_dev`, `cleartouch`, `currencycloud`,
    `cross_river`, `dc_bank`, `dwolla`, `evolve`, `goldman_sachs`, `iso20022`,
    `jpmc`, `mx`, `signet`, `silvergate`, `swift`, or `us_bank`.
    """


Balances = Balance
"""This type is deprecated and will be removed in a future release.

Please use Balance instead.
"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["PaymentActionUpdateParams"]


class PaymentActionUpdateParams(TypedDict, total=False):
    status: Required[Literal["pending", "processable", "processing", "sent", "acknowledged", "failed", "cancelled"]]
    """Optional.

    Set the status of the payment action to `cancelled` to cancel the payment
    action. This will only work if the payment action is in a `pending` state.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from .. import _legacy_response
from ..types import bulk_result_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import path_template, maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.bulk_result import BulkResult

__all__ = ["BulkResults", "AsyncBulkResults"]


class BulkResults(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BulkResultsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return BulkResultsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BulkResultsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return BulkResultsWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkResult:
        """
        get bulk_result

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/api/bulk_results/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BulkResult,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | Omit = omit,
        entity_id: str | Omit = omit,
        entity_type: Literal[
            "payment_order",
            "ledger_account",
            "ledger_transaction",
            "expected_payment",
            "transaction",
            "entity_link",
            "transaction_line_item",
            "bulk_error",
        ]
        | Omit = omit,
        per_page: int | Omit = omit,
        request_id: str | Omit = omit,
        request_type: Literal["bulk_request"] | Omit = omit,
        status: Literal["pending", "successful", "failed"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPage[BulkResult]:
        """
        list bulk_results

        Args:
          entity_id: Unique identifier for the result entity object.

          entity_type: The type of the request that created this result. bulk_request is the only
              supported `request_type`

          request_id: Unique identifier for the request that created this bulk result. This is the ID
              of the bulk request when `request_type` is bulk_request

          request_type: The type of the request that created this result. bulk_request is the only
              supported `request_type`

          status: One of successful or failed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/bulk_results",
            page=SyncPage[BulkResult],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "entity_id": entity_id,
                        "entity_type": entity_type,
                        "per_page": per_page,
                        "request_id": request_id,
                        "request_type": request_type,
                        "status": status,
                    },
                    bulk_result_list_params.BulkResultListParams,
                ),
            ),
            model=BulkResult,
        )


class AsyncBulkResults(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBulkResultsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBulkResultsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBulkResultsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncBulkResultsWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkResult:
        """
        get bulk_result

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/api/bulk_results/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BulkResult,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | Omit = omit,
        entity_id: str | Omit = omit,
        entity_type: Literal[
            "payment_order",
            "ledger_account",
            "ledger_transaction",
            "expected_payment",
            "transaction",
            "entity_link",
            "transaction_line_item",
            "bulk_error",
        ]
        | Omit = omit,
        per_page: int | Omit = omit,
        request_id: str | Omit = omit,
        request_type: Literal["bulk_request"] | Omit = omit,
        status: Literal["pending", "successful", "failed"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BulkResult, AsyncPage[BulkResult]]:
        """
        list bulk_results

        Args:
          entity_id: Unique identifier for the result entity object.

          entity_type: The type of the request that created this result. bulk_request is the only
              supported `request_type`

          request_id: Unique identifier for the request that created this bulk result. This is the ID
              of the bulk request when `request_type` is bulk_request

          request_type: The type of the request that created this result. bulk_request is the only
              supported `request_type`

          status: One of successful or failed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/bulk_results",
            page=AsyncPage[BulkResult],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "entity_id": entity_id,
                        "entity_type": entity_type,
                        "per_page": per_page,
                        "request_id": request_id,
                        "request_type": request_type,
                        "status": status,
                    },
                    bulk_result_list_params.BulkResultListParams,
                ),
            ),
            model=BulkResult,
        )


class BulkResultsWithRawResponse:
    def __init__(self, bulk_results: BulkResults) -> None:
        self._bulk_results = bulk_results

        self.retrieve = _legacy_response.to_raw_response_wrapper(
            bulk_results.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            bulk_results.list,
        )


class AsyncBulkResultsWithRawResponse:
    def __init__(self, bulk_results: AsyncBulkResults) -> None:
        self._bulk_results = bulk_results

        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            bulk_results.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            bulk_results.list,
        )


class BulkResultsWithStreamingResponse:
    def __init__(self, bulk_results: BulkResults) -> None:
        self._bulk_results = bulk_results

        self.retrieve = to_streamed_response_wrapper(
            bulk_results.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            bulk_results.list,
        )


class AsyncBulkResultsWithStreamingResponse:
    def __init__(self, bulk_results: AsyncBulkResults) -> None:
        self._bulk_results = bulk_results

        self.retrieve = async_to_streamed_response_wrapper(
            bulk_results.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            bulk_results.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PaymentActionCreateParams"]


class PaymentActionCreateParams(TypedDict, total=False):
    type: Required[str]
    """Required. The type of the payment action. Determines the action to be taken."""

    actionable_id: str
    """Optional. The ID of the associated actionable object."""

    actionable_type: str
    """Optional.

    The type of the associated actionable object. One of `payment_order`,
    `expected_payment`. Required if `actionable_id` is passed.
    """

    details: object
    """Optional. The specifc details of the payment action based on type."""

    internal_account_id: str
    """Optional.

    The ID of one of your organization's internal accounts. Required if
    `actionable_id` is not passed.
    """
